#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


"""
This file contains reporting functions.
"""

import json

from abc import ABCMeta, abstractmethod
from os.path import splitext
from xml.etree import ElementTree as ET


from .exceptions import PlecostInvalidReportFormat


# --------------------------------------------------------------------------
# Abstract
# --------------------------------------------------------------------------
class Reporter(object, metaclass=ABCMeta):
    """Reporter abstract class"""

    # ----------------------------------------------------------------------
    def __init__(self, output_filename):
        """
        :param output_filename: Output file name.
        :type output_filename: str
        """
        if not isinstance(output_filename, str):
            raise TypeError("Expected basestring, got '%s' instead" % type(output_filename))

        self.__output_filename = output_filename

    # ----------------------------------------------------------------------
    @property
    def output_filename(self):
        """
        :return: output file name.
        :rtype: str
        """
        return self.__output_filename

    # ----------------------------------------------------------------------
    # Abstract methods
    # ----------------------------------------------------------------------
    @abstractmethod
    def generate(self, info):
        """
        Generates content of report

        :param info: PlecostResults instance
        :type info: `PlecostResults`

        :return: content of report
        :rtype: object

        """
        raise NotImplemented()

    # ----------------------------------------------------------------------
    @abstractmethod
    def save(self, content):
        """
        Save the the content of report into output_file

        :param content: object with content
        :type content: object
        """
        raise NotImplemented()


# --------------------------------------------------------------------------
# Implementation
# --------------------------------------------------------------------------
class ReporterJSON(Reporter):
    """JSON reporter"""

    # ----------------------------------------------------------------------
    def generate(self, info):
        """
        Generates content of report

        :param info: PlecostResults instance
        :type info: `PlecostResults`
        """
        js_info = {}

        # Set target
        js_info["target"] = info.target

        # Set time info
        js_info["start_time"] = info.start_time.strftime("%H-%m-%Y %H:%M:%S")
        js_info["end_time"] = info.end_time.strftime("%H-%m-%Y %H:%M:%S")

        # WordPress info
        js_info["wordpress"] = {
            "current_version": info.wordpress_info.current_version,
            "last_version": info.wordpress_info.latest_version,
            "outdated": info.wordpress_info.is_outdated,
            "cves": [x for x in info.wordpress_info.vulnerabilities]
        }

        # Plugins info
        js_info["plugins"] = []
        for plugin in info.plugins:

            json_plugin = {}
            json_plugin["plugin_name"] = plugin.plugin_name

            json_plugin["current_version"] = plugin.current_version
            json_plugin["last_version"] = plugin.latest_version
            json_plugin["url"] = plugin.plugin_uri
            json_plugin["outdated"] = plugin.is_outdated

            # Set CVE
            json_plugin["cves"] = [cve for cve in plugin.cves]

            # Set exploits
            json_plugin["exploits"] = [exploit for exploit in plugin.exploits]

            js_info["plugins"].append(json_plugin)

        return js_info

    # ----------------------------------------------------------------------
    def save(self, content):
        # Save to file
        json.dump(content, open(self.output_filename, "w"))


# --------------------------------------------------------------------------
class ReporterXML(Reporter):
    """XML reporter"""

    # ----------------------------------------------------------------------
    def generate(self, info):
        """
        Generates content of report

        :param info: PlecostResults instance
        :type info: `PlecostResults`
        """
        root = ET.Element("libs")

        # Set target
        target = ET.SubElement(root, "target")
        target.text = info.target

        # Set time info
        time_start = ET.SubElement(root, "start_time")
        time_start.text = info.start_time.strftime("%H-%m-%Y %H:%M:%S")

        time_end = ET.SubElement(root, "end_time")
        time_end.text = info.end_time.strftime("%H-%m-%Y %H:%M:%S")

        # WordPress info
        wordpress = ET.SubElement(root, "wordpress")
        wordpress.set("current_version", info.wordpress_info.current_version)
        wordpress.set("last_version", info.wordpress_info.latest_version)

        # Set CVE
        if info.wordpress_info.vulnerabilities:
            cves = ET.SubElement(wordpress, "cves")
            for cve in info.wordpress_info.vulnerabilities:
                xml_cve = ET.SubElement(cves, "cve")
                xml_cve.text = cve

        # Plugins info
        plugins = ET.SubElement(root, "plugins")
        for plugin in info.plugins:
            xml_plugin = ET.SubElement(plugins, "plugin")
            xml_plugin.text = plugin.plugin_name

            xml_plugin.set("current_version", plugin.current_version)
            xml_plugin.set("last_version", plugin.latest_version)
            xml_plugin.set("url", plugin.plugin_uri)
            xml_plugin.set("outdated", "Yes" if plugin.is_outdated else "No")

            # Set CVE
            if plugin.cves:
                cves = ET.SubElement(xml_plugin, "cves")
                for cve in plugin.cves:
                    xml_cve = ET.SubElement(cves, "cve")
                    xml_cve.text = cve

            # Set exploits
            if plugin.cves:
                exploits = ET.SubElement(xml_plugin, "exploits")
                for exploit in plugin.exploits:
                    xml_exploit = ET.SubElement(exploits, "exploits")
                    xml_exploit.text = exploit

        return root

    # ----------------------------------------------------------------------
    def save(self, content):
        # Save to file
        tree = ET.ElementTree(content)
        tree.write(self.output_filename, encoding="UTF-8")


# ----------------------------------------------------------------------
def get_reporter(filename):
    """
    Select correct reporter by their extension.

    :param filename: file name path.
    :type filename: basestring

    :return: Reporter instance
    :rtype: `Reporter`
    """
    reporters = dict(xml=ReporterXML,
                     json=ReporterJSON)

    try:
        extension = splitext(filename)[1][1:]

        return reporters[extension]
    except KeyError:
        raise PlecostInvalidReportFormat("Report format '%s' not found." % extension)


__all__ = ["Reporter", "get_reporter"]


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


"""
This file contains function to looking for WordPress plugins and versions
"""

import csv
import aiohttp
import asyncio

from datetime import datetime
from functools import partial
from urllib.parse import urlparse
from os.path import join

from .db import DB
from .data import *  # noqa
from .exceptions import *  # noqa
from .plugins_utils import plugins_testing
from .utils import colorize, generate_error_page, download, get_data_folder
from .wordpress_core import is_remote_a_wordpress, get_wordpress_version, get_wordpress_vulnerabilities


# ----------------------------------------------------------------------
# Main code of functions
# ----------------------------------------------------------------------
def find_versions(args):
    """
    Main function to run libs as version finder.

    :param args: PlecostOptions object
    :type args: `PlecostOptions`

    :return: PlecostResults object.
    :rtype: `PlecostResults`

    :raises: PlecostTargetNotAvailable, PlecostNotWordPressFound
    """
    # --------------------------------------------------------------------------
    # Common vars
    # --------------------------------------------------------------------------
    url = args.target
    parsed_url = urlparse(args.target)
    host = parsed_url.hostname
    concurrency = args.concurrency
    log = args.log_function
    proxy = args.proxy
    is_color = args.colorize
    start_time = datetime.now()
    no_check_wordpress = args.no_check_wordpress
    no_check_plugins = args.no_check_plugins
    no_check_wordpress_version = args.no_check_wordpress_version
    force_scan = args.force_scan

    # Jackass mode is set?
    if args.jackass is True:
        concurrency = 9999

    # Non-blocking config
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    con = aiohttp.TCPConnector(conn_timeout=10, share_cookies=True, loop=loop, verify_ssl=False)
    _download = partial(download, max_redirect=0, connector=con, loop=loop)

    # Get CVE database
    db = DB(path=join(get_data_folder(), "cve.db"))

    # --------------------------------------------------------------------------
    # Test availability of target
    # --------------------------------------------------------------------------
    log("[*] Testing target connection...")
    headers, status, content = loop.run_until_complete(_download(url, method="get", get_content=False))

    # Detect redirect
    if status in (300, 301, 302, 303, 307):
        url = headers.get("location", None)

        if url is not None:
            log("\n[%s] Redirection detected to '%s'. Using it now. " % (colorize("ii", "yellow"), url),
                log_level=1)
        else:
            raise PlecostTargetNotAvailable("Redirection detected, but can't determinate the new location")
    log(colorize(" ok!\n"))

    # --------------------------------------------------------------------------
    # Check if remote host is a WordPress
    # --------------------------------------------------------------------------
    if no_check_wordpress is False:
            log("[*] Looking for WordPress installation...\n")
            # Error page content.
            headers, status, error_page = loop.run_until_complete(_download(generate_error_page(url)))

            _is_wp = loop.run_until_complete(is_remote_a_wordpress(url, error_page, _download))

            if not _is_wp:
                if force_scan is False:
                    raise PlecostNotWordPressFound("No WordPress installations found in '%s'." % host)
                else:
                    log(colorize("\n   No Wordpress installation found!\n", "yellow"))
            else:
                log("\n   %s" % colorize(" ok!\n"))

    # --------------------------------------------------------------------------
    # Check WordPress version
    # --------------------------------------------------------------------------
    if no_check_wordpress_version is False:
            log("[*] Getting WordPress version... ")

            wordpress_version = loop.run_until_complete(get_wordpress_version(url, _download, db))
            # wordpress_version.
            if wordpress_version:
                log("%s (latest: %s)" %
                    (
                        colorize("%s" % wordpress_version.current_version,
                                 "red" if wordpress_version.is_outdated is True else "blue"),
                        colorize("%s" % wordpress_version.latest_version)
                    ), 0)

                # --------------------------------------------------------------------------
                # Looking for CVEs for installed Wordpress version
                # --------------------------------------------------------------------------
                if wordpress_version.vulnerabilities:
                    log("\n    |_CVE list:\n")
                    for cve in wordpress_version.vulnerabilities:
                        log("    |__%(cve)s: (http://cve.mitre.org/cgi-bin/cvename.cgi?name=%(cve)s)\n" %
                            {"cve": colorize(cve, "red")})
                    log("\n")
            else:
                    log(colorize("Unknown!\n", "red"))

            log("\n")
    else:
        wordpress_version = PlecostWordPressInfo(last_version="",
                                                 current_version="",
                                                 vulnerabilities=[])

    # --------------------------------------------------------------------------
    # Check the plugins
    # --------------------------------------------------------------------------
    # Read plugins file and remove \n and \r
    plugins_info = []

    if no_check_plugins is False:
        plugins = []
        plugins_append = plugins.append

        with open(args.wordlist, "rU") as f:
            for plugin in f:
                plugins_append(plugin.replace("\n", "").replace("\r", ""))

        # Prepare csv file
        cve_info = csv.reader(plugins)
        error_page = ""
        # Find plugins
        log("[*] Looking for plugins (wordlist: %s) ... " % args.wordlist[args.wordlist.rfind("/") + 1:], 0)

        plugins_info = loop.run_until_complete(plugins_testing(url,
                                                               error_page,
                                                               log,
                                                               cve_info,
                                                               db,
                                                               concurrency,
                                                               loop,
                                                               con=con))
    log("\n[*] Done! \n")

    # Set finish time
    end_time = datetime.now()

    # --------------------------------------------------------------------------
    # Clean up
    # --------------------------------------------------------------------------
    con.close()

    # --------------------------------------------------------------------------
    # Make results
    # --------------------------------------------------------------------------
    return PlecostResults(target=args.target,
                          start_time=start_time,
                          end_time=end_time,
                          wordpress_info=wordpress_version,
                          plugins=plugins_info)


__all__ = ["find_versions", "_is_remote_a_wordpress"]


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

"""
This file contains API calls for all Plecost functions
"""

__version__ = "1.0.0"
__all__ = ["run", "find_versions", "PlecostOptions", "PlecostInvalidReportFormat"]

from sys import version_info

from .libs.reporters import *  # noqa
from .libs.data import PlecostOptions
from .libs.versions import find_versions  # noqa


# --------------------------------------------------------------------------
#
# Command line options
#
# --------------------------------------------------------------------------
def run(config):
    """
    Main function of libs:
    - Find WordPress versions
    - Find outdated plugins

    :param config: PlecostOptions option instance
    :type config: `PlecostOptions`

    :raises: PlecostTargetNotAvailable, PlecostNotWordPressFound, PlecostWordListNotFound
    """

    # --------------------------------------------------------------------------
    # Checks Python version
    # --------------------------------------------------------------------------
    if version_info < (3, 3):
        raise RuntimeError("You need Python 3.3.x or higher to run Plecost")

    # Check reporter
    if config.report_filename is not None:
        # Select appropriate report.
        reporter_function = get_reporter(config.report_filename)

    # Find wordpress and plugins versions
    data = find_versions(config)

    # Generate reports
    if config.report_filename is not None:

        report = reporter_function(config.report_filename)

        # Get report content
        content = report.generate(data)

        # Save report content
        report.save(content)


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


import re
import io
import gzip
import sqlite3

from os.path import join
from urllib.error import URLError
from urllib.request import urlopen
from collections import defaultdict
from xml.etree import ElementTree as ET

from ..utils import update_progress, colorize, get_data_folder
from ..db import DB


# ----------------------------------------------------------------------
def _insert_cves(cves, con):
    """
    Insert CVE
    """
    for cve, description in cves:
        try:
            con.execute("INSERT INTO CVE (cve, cve_description) VALUES (?, ?)", (cve, description))
        except sqlite3.IntegrityError:
            pass

        yield cve


# ----------------------------------------------------------------------
def _store_plugins_vulnerabilities_in_db(data, connection, log):
    """Comment"""

    for plugin_name, version_cv in data.items():

        for plugin_version, cves in version_cv.items():

            # Store plugin info
            try:
                c = connection.execute("INSERT INTO PLUGIN_VULNERABILITIES (plugin_name, plugin_version) VALUES(?, ?)",
                                       (plugin_name, plugin_version, ))
            except sqlite3.IntegrityError:
                log("Error integrity in plugin: (%s, %s)" % (plugin_name, plugin_version), log_level=3)
                continue

            # Store CVEs
            for cve in _insert_cves(cves, connection):
                # Store relations
                connection.execute("INSERT INTO PLUGIN_VULNERABILITIES_CVE VALUES(?, ?)", (c.lastrowid, cve, ))

    connection.commit()


# ----------------------------------------------------------------------
def _store_wordpress_vulnerabilities_in_db(data, connection, log):
    """Comment"""

    for wordpress_version, cves in data.items():

        # Store plugin info
        try:
            connection.execute("INSERT INTO WORDPRESS_VULNERABILITIES VALUES(?)", (wordpress_version, ))
        except sqlite3.IntegrityError:
            log("Error integrity in wordpress version: %s" % wordpress_version, log_level=3)
            continue

        # Store CVEs
        for cve in _insert_cves(cves, connection):
            # Store relations
            connection.execute("INSERT INTO WORDPRESS_VULNERABILITIES_CVE VALUES(?, ?)", (wordpress_version, cve, ))

    connection.commit()


# ----------------------------------------------------------------------
def _generate_previous_versions(version):
    """
    Generates all versions from current to back.

    :param version: version in format: x.y.z
    :type version: str

    :return: a list with versions
    :rtype: list(str)

    """
    if not isinstance(version, str):
        return []

    res = []
    res_append = res.append

    _splited_version = version.split(".")

    if len(_splited_version) > 2:
        # Get version part: 3.0.1 -> 3.0
        _version = "%s.%s." % (_splited_version[0], _splited_version[1])

        # Get release part: 3.0.1 -> 1, and checks that is integer
        try:
            _release = int(_splited_version[2])
        except ValueError:
            return []

        if _release == 0:
            return []

        # Generate verstion 0 -> current
        for x in range(0, _release + 1):
            res_append("%s%s" % (_version, x))

        return res

    else:
        return []


# ----------------------------------------------------------------------
def _parse_vulnerabilities_from_nvd(stream, log=None, cpe=None):
    """
    Get NVD xml path, and return plugins name and return dict with info:

    Return tuple: plugins_vulns, wordpress_vulns

    Where:

    - Plugin_vulns: { PLUGIN_NAME: {VERSION: [CVE] } +
    - Wordpress_vulns: { VERSION: [CVE] }

    :return: Tuple plugins_vulns, wordpress_vulns
    :rtype: dict, dict

    """
    regex = re.compile("([\d]*\.[\d]+\.*[\d]*)")

    plugins = {}
    wordpress = defaultdict(list)

    log("        Processing file: ")

    parsed = ET.fromstring(stream)

    for x in update_progress(parsed.getchildren(), prefix_text="        Processing file: "):

        _tmp_v = x.findall(".//{http://cpe.mitre.org/language/2.0}fact-ref")

        # Parse version
        if _tmp_v is not None:

            for v in _tmp_v:
                _v_name = v.get("name")
                if "~~~wordpress~~" in _v_name or ":wordpress:" in _v_name:
                    _v_name_split = _v_name.split(":")

                    if len(_v_name_split) > 3:
                        _product = _v_name_split[3]
                    else:
                        continue

                    if len(_v_name_split) > 4:
                        _version = _v_name.split(":")[4]
                    else:
                        continue

                    if regex.match(_version):

                        cve_id = x.find(".//{http://scap.nist.gov/schema/vulnerability/0.4}cve-id").text
                        cve_description = x.find(".//{http://scap.nist.gov/schema/vulnerability/0.4}summary").text

                        # Wordpress vuln
                        if _product == "wordpress":
                            wordpress[_version].append((cve_id, cve_description))

                            # generate previous versions
                            for v in _generate_previous_versions(_version):
                                wordpress[v].append((cve_id, cve_description))

                            if _version == "3.9.3":
                                print(_version)

                        else:
                            # Plugin vulns
                            try:
                                plugins[_product][_version].append((cve_id, cve_description))
                            except KeyError:
                                plugins[_product] = defaultdict(list)
                                plugins[_product][_version].append((cve_id, cve_description))

                            # generate previous versions
                            for v in _generate_previous_versions(_version):
                                plugins[_product][v].append((cve_id, cve_description))

    return plugins, wordpress


# --------------------------------------------------------------------------
def update_cve(log, since=2013):
    """
    Generate the CVE with WordPress vulns related DB and store it into pickled database.

    :param log: Log function as format: function(Message, Level)
    :type log: function(str, int)

    :param since: get NVD since this date
    :type since: int

    :return: None
    """
    if not isinstance(since, int):
        raise TypeError("Expected int, got '%s' instead" % type(since))

    nvd_base_url = "https://nvd.nist.gov/feeds/xml/cve/nvdcve-2.0-%s.xml.gz"

    log("[*] Updating CVE database...\n")

    # Create database
    db = DB(join(get_data_folder(), "cve.db"), auto_create=False)
    db.clean_db()
    db.create_db()

    # return
    for d in range(since, 2016):
        log("    Downloading NVD feed %s: \n" % d)

        _nvd_url = nvd_base_url % d

        # --------------------------------------------------------------------------
        # Obtain CVE database
        # --------------------------------------------------------------------------
        try:
            content = urlopen(_nvd_url).read()
        except URLError as e:
            log("[%s] %s" % (
                colorize("!"),
                colorize("Can't obtain CVE database.")
            ))
            log("    |- Error details: %s" % e, 3)

        # Load
        unziped_content = gzip.GzipFile(fileobj=io.BytesIO(content)).read()

        # Parse info
        p, w = _parse_vulnerabilities_from_nvd(unziped_content, log)

        log("\n")

        # Store
        _store_plugins_vulnerabilities_in_db(p, db.con, log)
        _store_wordpress_vulnerabilities_in_db(w, db.con, log)

    log("\n[*] Done!\n")
